from models.db import db_conn
import settings

class Model():
    def __init__(self):
        self.collection_name = None
        self.allowed_params = None
        self.unique_key = None
        self.attributes = None
        self.list_attributes = None

    def fetch_by_unique_keys(self, unique_key_values):
        """
        Fetch multiple records by unique key identifier.

        :param(list of str) unique_key_values: List of unique values
        :return(int, list of dicts): List of record dicts
        """
        query_dict = {
            self.unique_key: {'$in': unique_key_values}
        }
        results = db_conn[self.collection_name].find(filter=query_dict)
        serialized_records = [self.serialize_record(rec) for rec in results]
        return serialized_records

    def fetch_all(self, **kwargs):
        """
        Fetch multiple pages of records matching query parameter and sorted as well

        :param(dict) kwargs: Key-value pair for forming search query
            Keys could any of the "allowed_params" or any of the following ones:
                1. sort: Comma-separated string for attributes preceded by + or -
                2. page: Page number (Pagination)
                3. size: Size of pages (Pagination)
                4. query: String for text search
        :return(list of dict):
        """
        query_dict = {}

        # Create usual filter parameters
        for key, value in kwargs.items():
            if key not in self.allowed_params:
                raise AttributeError("{} is not an allowed param".format(key))
            # These 4 to be treated separately
            if key in ['page', 'size', 'query', 'sort']:
                continue
            query_dict[key] = self.allowed_params[key](value)
        # Query Text Search
        if 'query' in kwargs:
            query_dict['$text'] = {'$search': kwargs['query']}

        # print(query_dict)
        if 'species_tax_id' in kwargs:
            if query_dict['species_tax_id'] == 'all':
                del query_dict['species_tax_id']
            elif query_dict['species_tax_id'] == 'null':
                query_dict['species_tax_id'] = None
            results = db_conn[self.collection_name].find(query_dict, self.list_attributes)
        else:
            results = db_conn[self.collection_name].find(query_dict)

        total_count = db_conn[self.collection_name].count(query_dict)
        #results = db_conn[self.collection_name].find(query_dict)
        #total_count = db_conn[self.collection_name].count(query_dict)
        #print(query_dict)
        # print(total_count)

        # Pagination stuff
        # prev_page_no = None
        # next_page_no = None
        # if total_count > 0 and ('page' in kwargs or 'size' in kwargs):
        #     # print(kwargs)
        #     page_no = int(kwargs.get('page', 1))
        #     size = int(kwargs.get('size', settings.DEFAULT_PAGE_SIZE))
        #     skip_value = (page_no - 1) * size
        #     if skip_value >= total_count:
        #         raise ValueError('Value for page_no and size exceeded total count {}'.format(total_count))
        #     if page_no > 1:
        #         prev_page_no = page_no - 1
        #     if skip_value + size < total_count:
        #         next_page_no = page_no + 1
        #     results = results.skip(skip_value).limit(size)

        # Sorting
        # sort = []
        # if 'sort' in kwargs:
        #     for field in kwargs['sort'].split(','):
        #         direction = 1
        #         if field.startswith('-'):
        #             field = field[1:]
        #             direction = -1
        #         sort.append((field, direction))
        #     results = results.sort(sort)

        serialized_records = [self.serialize_record(rec) for rec in results]

        # return total_count, serialized_records, prev_page_no, next_page_no
        return total_count, serialized_records

    def fetch_record(self, unique_id):
        """
        Fetch single record by unique ID

        :param(str) unique_id:
        :return(dict):
        """
        return self.serialize_record(db_conn[self.collection_name].find_one({self.unique_key: unique_id}))

    def serialize_record(self, record):
        """
        Serialize database object to dictionary
        Only add allowed attributes

        :param(dict) record:
        :return(dict):
        """
        response = {}
        for key in self.attributes:
            response[key] = record.get(key)
        return response
